
'-----------------------------------------------------------------------
'
' ***  Capacitive Touch Keypad ***
'
' Description: Simple capacitive touch keypad with 4 key
'              - automatic calibration
'              - output: active Low or active High
'              - touchpad area 16x16 mm
'
' Copyright: Quercus Lab 2012
'
' Author: Ivica Vlasic
'
' Hardware:
'
' CHANNEL 1
' Vcc-----------
'              |
'              -
'             | |
'              - 680k
'              |
'PortD.3-------+---- Touchpad 16x16 mm
'
' CHANNEL 2
' Vcc-----------
'              |
'              -
'             | |
'              - 680k
'              |
'PortD.2-------+---- Touchpad 16x16 mm
'
' CHANNEL 3
' Vcc-----------
'              |
'              -
'             | |
'              - 680k
'              |
'PortD.5-------+---- Touchpad 16x16 mm
'
' CHANNEL 4
' Vcc-----------
'              |
'              -
'             | |
'              - 680k
'              |
'PortD.4-------+---- Touchpad 16x16 mm

'-----------------------------------------------------------------------


$baud = 19200
$crystal = 8000000
$regfile = "attiny2313a.dat"
$hwstack = 32                                               ' default use 32 for the hardware stack
$swstack = 32                                               ' default use 10 for the SW stack
$framesize = 32

config portb = output

declare sub InitTouch()

dim TouchTreshold as integer
dim PB_State as bit
dim PB_Invert as bit


PB_Rate alias PINd.0 : portd.0 = 1
PB_OUT alias Pind.1 : portd.1 = 1

LED1 alias portb.6
LED2 alias Portb.7
LED3 alias PORTB.5
LED4 alias PORTb.4

PushButton1 alias Portb.0
PushButton2 alias Portb.1
PushButton3 alias Portb.2
PushButton4 alias Portb.3

TouchTreshold = 0

' read configuration on start
PB_State = PB_Out
PB_Invert = PB_State : toggle PB_Invert

' test output LEDs
portb = 255 : wait 1
portb = 0 : wait 1
set LED1 : wait 1
set LED2 : wait 1
set LED3 : wait 1
set LED4 : wait 1
portb = 0

call InitTouch()

' main loop
do

   ' Channel 1
   DDRD.3 = 1 : Portd.3 = 0
   waitms 2
   Portd.3 = 0 : DDRd.3 = 0
   waitus TouchTreshold
   if pind.3 = 0 then
     set LED1
     PushButton1 = PB_State
   else
     reset LED1
     PushButton1 = PB_Invert
   endif

   ' Channel 2
   DDRd.2 = 1 : Portd.2 = 0
   waitms 2
   Portd.2 = 0 : DDRd.2 = 0
   waitus TouchTreshold
   if pind.2 = 0 then
     set LED2
     PushButton2 = PB_State
   else
     reset LED2
     PushButton2 = PB_Invert
   endif

   ' Channel 3
   DDRd.5 = 1 : Portd.5 = 0
   waitms 2
   Portd.5 = 0 : DDRd.5 = 0
   waitus TouchTreshold
   if pind.5 = 0 then
     set LED3
     PushButton3 = PB_State
   else
     reset LED3
     PushButton3 = PB_Invert
   endif

    ' Channel 4
   DDRd.4 = 1 : Portd.4 = 0
   waitms 2
   Portd.4 = 0 : DDRd.4 = 0
   waitus TouchTreshold
   if pind.4 = 0 then
     set LED4
     PushButton4 = PB_State
   else
     reset LED4
     PushButton4 = PB_Invert
   endif

   waitms 100
loop


' automatic calibration subroutine
sub InitTouch()

 do
   TouchTreshold = TouchTreshold + 1
   DDRD.3 = 1 : portD.3 = 0
   waitms 2

   PortD.3 = 0 : DDRd.3 = 0
   waitus TouchTreshold
   if pinD.3 = 1 then
     TouchTreshold = TouchTreshold + 2
     exit do
   endif
 loop

end sub